#include <AT89X55.H>
#include <rtx51tny.h>

///////////////////////////////////////////////////////////////////////////////////////
// Shared variables and declarations between the music task and the serial task
///////////////////////////////////////////////////////////////////////////////////////
typedef struct _commands {
	unsigned char cmd_type;          // Command Type: 0 == tempo
	                                 //               1 == frequency / chords

	unsigned char val;               // Interpretation of val is dependent upon cmd_type
	unsigned char id1;
	unsigned char note1;
	unsigned char id2;
	unsigned char note2;
	unsigned char id3;
	unsigned char note3;
	unsigned char id4;
	unsigned char note4;
} command;

#define MAX_CMDS       1             // The maximum number of CMDS in the CMDS queue.
#define NEXT_PTR(x)    (( x >= (MAX_CMDS)) ? 0 : x + 1)

/*
 * Queue variables, we are implementing it as a circular queue
 * there are two definitions of the deq_cmd and enq_cmd which are dependent upon
 * the number of MAX_CMDS in the circular queue. They are very different as it
 * turns out that if MAX_CMDS == 1, there is less memory requirements.
 * So I took advantage of this.
 */
#if (MAX_CMDS > 1)
	command cmd_array[MAX_CMDS];
	unsigned char cmd_head = 0;
	unsigned char cmd_tail = 0;
	command temp_cmd;
	command cur_cmd;
	
	/*
	 * Accessor functions for the queue.
	 * they will return 0 if there was an error
	 * and 1 if it succeeded.
	 */
	bit deq_cmd() {
		if (cmd_head == cmd_tail)
			return 0;
	
		cur_cmd = cmd_array[cmd_tail];
	
		ET0 = 0;
		cmd_tail = NEXT_PTR(cmd_tail);
		ET0 = 1;
	
		return 1;
	}
	
	// this will take what ever is temp_cmd and dump it into the queue.
	bit enq_cmd() {
		if (cmd_tail == NEXT_PTR(cmd_head))
			return 0;
	
		cmd_array[cmd_head] = temp_cmd;
	
		ET0 = 0;
		cmd_head = NEXT_PTR(cmd_head);
		ET0 = 1;
	
		return 1;
	}
#else // MAX_CMDS <= 1;

	command temp_cmd;
	command queue;
	command cur_cmd;
	bit cmd_mutex = 0;
	
	bit deq_cmd() {
		if (cmd_mutex == 0)  // There is nothing to return.
			return 0;

		//EA = 0;
	
		cur_cmd.cmd_type = queue.cmd_type;
		cur_cmd.val = queue.val;
		cur_cmd.id1 = queue.id1;
		cur_cmd.id2 = queue.id2;
		cur_cmd.id3 = queue.id3;
		cur_cmd.id4 = queue.id4;
		cur_cmd.note1 = queue.note1;
		cur_cmd.note2 = queue.note2;
		cur_cmd.note3 = queue.note3;
		cur_cmd.note4 = queue.note4;

		cmd_mutex = 0;

		//EA = 1;

		return 1;
	}
	
	
	bit enq_cmd() {
		if (cmd_mutex == 1) // We can't enque, because nobody has dequed.
			return 0;
	
		//EA = 0;

		queue.cmd_type = temp_cmd.cmd_type;
		queue.val = temp_cmd.val;
		queue.id1 = temp_cmd.id1;
		queue.id2 = temp_cmd.id2;
		queue.id3 = temp_cmd.id3;
		queue.id4 = temp_cmd.id4;
		queue.note1 = temp_cmd.note1;
		queue.note2 = temp_cmd.note2;
		queue.note3 = temp_cmd.note3;
		queue.note4 = temp_cmd.note4;

		cmd_mutex = 1;

		//EA = 1;

		return 1;
	}

#endif // Finish special cases.


////////////////////////////////////////////////////////////////////////
// Serial Specific operations.

/*
 * the following #defines are used to parse the bytes that are recieved
 */
#define CMD_TYPE(x)   ((x & 0x80) >> 7)
#define VAL(x)        (x & 0x7F)

#define ID1_VAL(x)       (x >> 6)
#define NOTE1_VAL_UP(x)  ((x & 0x3F) << 2)
#define NOTE1_VAL_DWN(x) ((x & 0xC0) >> 6)

#define ID2_VAL(x)       ((x & 0x30) >> 4)
#define NOTE2_VAL_UP(x)  ((x & 0x0F) << 4)
#define NOTE2_VAL_DWN(x) ((x & 0xF0) >> 4)

#define ID3_VAL(x)       ((x & 0x0C) >> 2)
#define NOTE3_VAL_UP(x)  ((x & 0x03) << 6)
#define NOTE3_VAL_DWN(x) ((x & 0xFC) >> 2)

#define ID4_VAL(x)       (x & 0x03)
#define NOTE4_VAL(x)     (x)

/*
 * serial state variables and state definitions
 */
#define RETURN_CHAR  'A'		// This is the return character that tells the sender
								// we successfully grabbed a byte.

#define SERIAL_STATE_START  0
#define SERIAL_STATE_TEMPO  1
#define SERIAL_STATE_VOICE  2

#define MAX_TEMPO_PACKETS 1     // The size of the Tempo Command in bytes.
#define MAX_VOICE_PACKETS 5     // The size of an entire voice command in bytes.

unsigned char serial_state;
unsigned char num_read;
bit new_serial_val;



///////////////////////////////////////////////////////////////////////////////////////
// Music Task Specific declarations
///////////////////////////////////////////////////////////////////////////////////////
#define MAX_INTERRUPT_CNT   4
#define TIMER2_RELOAD_TOP   0xFF
#define TIMER2_RELOAD_BOT   0x00

unsigned char scale_factor = 1;
unsigned int tne_counter=0;
unsigned int tne = 60000;
unsigned char temp;
unsigned char tempo = 0;


///////////////////////////////////////////////////////////////////////////////////////
// task descriptions
///////////////////////////////////////////////////////////////////////////////////////
#define SERIAL_BOT_TASK     0

#define MUSIC_BOT_TASK      1
#define DEBUG_DMP           2
#define INIT_TASK           3

/////////////////////////////////////////////////////////////////////////////////////////
//  WAVE DEFINITIONS
/////////////////////////////////////////////////////////////////////////////////////////
code unsigned char sine[256] = 
{
127,130,133,136,139,143,146,149,152,155,
158,161,164,167,170,173,176,178,181,184,
187,190,192,195,198,200,203,205,208,210,
212,215,217,219,221,223,225,227,229,231,
232,234,235,238,239,240,242,243,244,245,
247,248,249,250,251,252,252,253,253,253,
253,254,254,254,254,254,254,254,253,253,
253,252,252,251,250,249,249,248,247,245,
244,243,242,240,239,238,236,234,233,231,
229,227,225,223,221,219,217,215,212,210,
208,205,203,200,198,195,192,190,187,184,
181,178,176,173,170,167,164,161,158,155,
152,149,146,143,139,136,133,130,127,124,
121,118,115,111,108,105,102, 99, 96, 93,
90,87,84,81,78,76,73,70,67,64,
62,59,56,54,51,49,46,44,42,39,
37,35,33,31,29,27,25,23,21,20,
18,16,15,14,12,11,10,9,7,6,
5,5,4,3,2,2,1,1,1,0,
0,0,0,0,0,0,1,1,1,2,
2,3,4,5,5,6,7,9,10,11,
12,14,15,16,18,20,21,23,25,27,
29,31,33,35,37,39,42,44,46,49,
51,54,56,59,62,64,67,70,73,76,
78,81,84,87,90,93,96,99,102,105,
108,111,115,118,121,124
};


code unsigned char square[256] =
{
127,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,

255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,127,0,0,

0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,

0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,

0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,

0,0,0,0,0,0
};


code unsigned char triangle[256] = {
127, 129, 131, 133, 135, 137, 139, 141, 143, 145, 147,
149, 151, 153, 155, 157, 159, 161, 163, 165, 167, 169,
171, 173, 175, 177, 179, 181, 183, 185, 187, 189, 191,
193, 195, 197, 199, 201, 203, 205, 207, 209, 211, 213,
215, 217, 219, 221, 223, 225, 227, 229, 231, 233, 235,
237, 239, 241, 243, 245, 247, 249, 251, 253, 255, 253,
251, 249, 247, 245, 243, 241, 239, 237, 235, 233, 231,
229, 227, 225, 223, 221, 219, 217, 215, 213, 211, 209,
207, 205, 203, 201, 199, 197, 195, 193, 191, 189, 187,
185, 183, 181, 179, 177, 175, 173, 171, 169, 167, 165,
163, 161, 159, 157, 155, 153, 151, 149, 147, 145, 143,
141, 139, 137, 135, 133, 131, 129, 127, 125, 123, 121,
119, 117, 115, 113, 111, 109, 107, 105, 103, 101,  99,
 97,  95,  93,  91,  89,  87,  85,  83,  81,  79,  77,
 75,  73,  71,  69,  67,  65,  63,  61,  59,  57,  55,
 53,  51,  49,  47,  45,  43,  41,  39,  37,  35,  33,
 31,  29,  27,  25,  23,  21,  19,  17,  15,  13,  11,
  9,   7,   5,   3,   1,   3,   5,   7,   9,  11,  13,
 15,  17,  19,  21,  23,  25,  27,  29,  31,  33,  35,
 37,  39,  41,  43,  45,  47,  49,  51,  53,  55,  57,
 59,  61,  63,  65,  67,  69,  71,  73,  75,  77,  79,
 81,  83,  85,  87,  89,  91,  93,  95,  97,  99, 101,
103, 105, 107, 109, 111, 113, 115, 117, 119, 121, 123,
125, 127, 127
};

code unsigned char saw_tooth10[256] = {
255, 254, 253, 252, 251, 250, 249, 248, 247, 246, 245,
244, 243, 242, 241, 240, 239, 238, 237, 236, 235, 234,
233, 232, 231, 230, 229, 228, 227, 226, 225, 224, 223,
222, 221, 220, 219, 218, 217, 216, 215, 214, 213, 212,
211, 210, 209, 208, 207, 206, 205, 204, 203, 202, 201,
200, 199, 198, 197, 196, 195, 194, 193, 192, 191, 190,
189, 188, 187, 186, 185, 184, 183, 182, 181, 180, 179,
178, 177, 176, 175, 174, 173, 172, 171, 170, 169, 168,
167, 166, 165, 164, 163, 162, 161, 160, 159, 158, 157,
156, 155, 154, 153, 152, 151, 150, 149, 148, 147, 146,
145, 144, 143, 142, 141, 140, 139, 138, 137, 136, 135,
134, 133, 132, 131, 130, 129, 128, 127, 126, 125, 124,
123, 122, 121, 120, 119, 118, 117, 116, 115, 114, 113,
112, 111, 110, 109, 108, 107, 106, 105, 104, 103, 102,
101, 100,  99,  98,  97,  96,  95,  94,  93,  92,  91,
 90,  89,  88,  87,  86,  85,  84,  83,  82,  81,  80,
 79,  78,  77,  76,  75,  74,  73,  72,  71,  70,  69,
 68,  67,  66,  65,  64,  63,  62,  61,  60,  59,  58,
 57,  56,  55,  54,  53,  52,  51,  50,  49,  48,  47,
 46,  45,  44,  43,  42,  41,  40,  39,  38,  37,  36,
 35,  34,  33,  32,  31,  30,  29,  28,  27,  26,  25,
 24,  23,  22,  21,  20,  19,  18,  17,  16,  15,  14,
 13,  12,  11,  10,   9,   8,   7,   6,   5,   4,   3,
  2,   1,   0
};

code unsigned char saw_tooth01[256] = {
  0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
 11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
 22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
 33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
 44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
 55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
 66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
 77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,
 88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,
 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120,
121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131,
132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142,
143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153,
154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164,
165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186,
187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197,
198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208,
209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219,
220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230,
231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241,
242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 
253, 254, 255
};

code unsigned char steps_whole[128] = {
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   2,   2,   2,   2,   2,
  2,   2,   3,   3,   3,   3,   3,   4,   4,   4,   4,   5,   5,   5,   6,
  6,   6,   7,   7,   8,   8,   9,   9,  10,  10,  11,  12,  12,  13,  14,
 15,  16,  17,  18,  19,  20,  21,  22,  24,  25,  27,  28,  30,  32,  34,
 36,  38,  40,  43,  45,  48,  51,  54,  57,  61,  64,  68,  72,  77,  81,
 86,  91,  97, 103, 109, 115, 122, 129, 137, 145, 154, 163, 173, 183, 194,
206, 218, 231, 245, 259, 0,0,0//275, 291,309
};


code unsigned int steps_dec1[128] = {
2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 4,
4, 4, 4, 5, 5, 5, 6, 6, 6, 7, 7, 8, 8,
9, 9, 0, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8,
9, 0, 1, 2, 4, 5, 7, 8, 0, 2, 4, 6, 8,
0, 2, 5, 8, 1, 4, 7, 0, 4, 8, 2, 6, 1,
5, 1, 6, 2, 8, 4, 1, 8, 6, 4, 3, 2, 1,
2, 2, 4, 6, 9, 3, 7, 2, 9, 6, 4, 3, 4,
6, 9, 3, 9, 6, 5, 6, 8, 2, 9, 7, 8, 2,
8, 6, 8, 3, 0, 2, 7, 6, 8, 6, 8, 4, 6,
4, 7, 6, 2, 4, 4, 2, 8, 2, 6, 0
};


code unsigned int steps_dec2[128] = {
0, 1, 2, 3, 5, 6, 8, 0, 1, 3, 5, 8, 0,
2, 5, 7, 0, 3, 6, 0, 3, 7, 1, 5, 0, 5,
0, 5, 1, 7, 3, 0, 7, 5, 3, 1, 0, 0, 0,
1, 2, 4, 7, 1, 5, 0, 6, 3, 1, 1, 1, 2,
5, 9, 5, 2, 1, 1, 3, 7, 3, 2, 2, 5, 1,
9, 0, 4, 2, 2, 7, 5, 7, 4, 5, 1, 2, 9,
1, 9, 4, 6, 5, 1, 6, 9, 1, 3, 6, 9, 3,
0, 0, 3, 1, 4, 3, 0, 4, 9, 3, 9, 9, 2,
1, 8, 4, 0, 9, 2, 2, 0, 9, 2, 0, 7, 6,
0, 1, 4, 2, 8, 8, 5, 4, 9, 7, 2
};


code unsigned int steps_dec3[128] = {
1, 3, 6, 9, 3, 8, 4, 1, 9, 8, 8, 0, 2,
6, 1, 8, 7, 7, 9, 3, 8, 6, 7, 9, 4, 2,
3, 7, 2, 4, 8, 5, 7, 3, 3, 9, 9, 5, 6,
3, 7, 8, 6, 1, 4, 6, 7, 8, 9, 0, 3, 8,
4, 7, 2, 3, 0, 4, 6, 7, 8, 1, 7, 7, 2,
5, 6, 8, 1, 9, 4, 6, 9, 5, 7, 7, 8, 3,
5, 9, 7, 3, 2, 7, 3, 5, 9, 9, 1, 2, 8,
5, 2, 4, 2, 3, 6, 1, 9, 0, 5, 8, 0, 5,
9, 5, 1, 4, 2, 4, 0, 3, 5, 2, 8, 0, 9,
4, 8, 5, 3, 9, 5, 3, 1, 6, 1, 0
};


//used for the decimal parts of a step
code unsigned char zero_tenth[10] = {0,0,0,0,0,0,0,0,0,0};
code unsigned char one_tenth[10] = {1,0,0,0,0,0,0,0,0,0};
code unsigned char two_tenth[10] = {1,0,0,0,0,1,0,0,0,0};
code unsigned char three_tenth[10] = {1,0,0,1,0,0,0,1,0,0};
code unsigned char four_tenth[10] = {1,0,0,1,0,1,0,0,1,0};
code unsigned char five_tenth[10] = {1,0,1,0,1,0,1,0,1,0};
code unsigned char six_tenth[10] = {1,0,1,0,1,0,1,0,1,1};
code unsigned char seven_tenth[10] = {1,0,1,1,1,0,1,0,1,1};
code unsigned char eight_tenth[10] = {1,0,1,1,1,0,1,1,1,1};
code unsigned char nine_tenth[10] = {1,1,1,1,1,0,1,1,1,1};


/////////////////////////////////////////////////////////////////////////////////////////
//  Variable Delcarations and #defines for the channels.
/////////////////////////////////////////////////////////////////////////////////////////
#define NUM_VOICES 4
#define VOICE_0 0
#define VOICE_1 1
#define VOICE_2 2
#define VOICE_3 3
#define VOICE_UNDEF  4

#define NUM_CHANNELS 4
#define CHANNEL_0 0
#define CHANNEL_1 1
#define CHANNEL_2 2
#define CHANNEL_3 3
#define NULL 0

typedef struct{
	unsigned char wave_id;
}voice_def;

typedef struct {
	unsigned char whole;
	unsigned char dec1;
	unsigned char dec1_index;
	unsigned char dec2;
	unsigned char dec2_index;
	unsigned char dec21_index;
}step;

typedef struct {
	step stp;
	unsigned char position;
	unsigned char voice_id;
}channel;

unsigned char getStepSize(unsigned char idx);
void setStep(unsigned char idx, unsigned char index);

//the array of all the voice descriptions
voice_def voices[NUM_VOICES];
channel channels[NUM_CHANNELS];





//////////////////////////////////////////////////////////////////////////////////////
// Interrupt Routines are as follows
//////////////////////////////////////////////////////////////////////////////////////

/*
 * serial_top
 * This is the serial interrupt routine.
 */
void serial_top (void) interrupt 4 {
	if (RI) {
		new_serial_val = 1;
		RI = 0;
		isr_send_signal(SERIAL_BOT_TASK);
	}

	if (TI) {
		TI = 0;
	}
}


/*
 * music_top
 * This will be set to timer 2 and interrupt every 1/16th of a second
 */

void music_top (void) interrupt 5 {
	unsigned char i;
	unsigned char length;
	temp =0;

	for(i=0;i<NUM_CHANNELS;i++){
	//if this voice is not the NULL voice
		if(channels[i].voice_id != VOICE_UNDEF){
			//length = getStepSize(i);
			length = getStepSize(i);
			(channels[i].position) +=  length;

			switch (voices[channels[i].voice_id].wave_id) {
			case 1:
				temp += sine[channels[i].position]/scale_factor;
				break;
			case 2:
				temp += triangle[channels[i].position]/scale_factor;
				break;
			case 3:
				temp += saw_tooth10[channels[i].position]/scale_factor;
				break;
			case 4:
				temp += square[channels[i].position]/scale_factor;
				break;
			default:
				temp += sine[channels[i].position]/scale_factor;
			}
		}
	}


	RD = 0;//set write line on dac
	P1 = temp;
	RD = 1;
	tne_counter++;
	if(tne_counter >= (tne>>tempo)){
		tne_counter = 0;
		isr_send_signal(MUSIC_BOT_TASK);
	}
	
	// Clear the Timer 2 interrupts.
	TF2 = 0;
}

////////////////////////////////////////////////////////////////////////////////////
// Task Implementations are as follows.
////////////////////////////////////////////////////////////////////////////////////

void music_bottom(void) _task_ MUSIC_BOT_TASK {
	unsigned char i;

	while (1) {
		INT0 = 0;

		os_wait(K_SIG, 0, 0);

		//it is bad news if we are in here this means that we don't have any music to play.	
		while(!deq_cmd()) {
			channels[0].voice_id = VOICE_UNDEF;
			channels[1].voice_id = VOICE_UNDEF;
			channels[2].voice_id = VOICE_UNDEF;
			channels[3].voice_id = VOICE_UNDEF;

			// Make sure to reset TNE.
			tne = 0;

			os_wait(K_SIG, 0, 0);
		}
	

		INT0 = 1;

		if(cur_cmd.cmd_type){
		
			//set up channel0	
			if(cur_cmd.id1){
				channels[0].voice_id = cur_cmd.id1;
				channels[0].position = 0;
				setStep(0, cur_cmd.note1);	
			}
			else
				channels[0].voice_id = VOICE_UNDEF;
			
			//set up channel1
			if(cur_cmd.id2){
				channels[1].voice_id = cur_cmd.id2;
				channels[1].position = 0;
				setStep(1, cur_cmd.note2);
			}
			else
				channels[1].voice_id = VOICE_UNDEF;
			

			//set up channel2
			if(cur_cmd.id3){
				channels[2].voice_id = cur_cmd.id3;
				channels[2].position = 0;
				setStep(2,cur_cmd.note3);
			}
			else
				channels[2].voice_id = VOICE_UNDEF;
	

			//set up channel3
			if(cur_cmd.id4){
				channels[3].voice_id = cur_cmd.id4;
				channels[3].position = 0;
				setStep(3,cur_cmd.note4);
			}
			else
				channels[3].voice_id = VOICE_UNDEF;
		
	
			tne =  cur_cmd.val << 9 ;//a tne of 1 should be about 256/10400 second about 1/32
		}else{
			tempo = cur_cmd.val;
		}
		//calculate the new scale factor
		
		scale_factor = 0;
		for(i=0;i<NUM_CHANNELS;i++){
			if(channels[i].voice_id != VOICE_UNDEF)
				scale_factor++;
		}

	

		os_send_signal(SERIAL_BOT_TASK);
	}
}



void serial_bottom(void) _task_ SERIAL_BOT_TASK {
	unsigned char tmp;
		// Initialize the Serial Port
	// We are using a 32 MHz clock. 
    SCON  = 0x50;		          /* SCON: mode 1, 8-bit UART, enable rcvr      */
    TMOD |= 0x20;                 /* TMOD: timer 1, mode 2, 8-bit reload        */
    TH1   = 221;                  /* TH1:  reload value for 2400 baud @ 32MHz   */
    TR1   = 1;                    /* TR1:  timer 1 run                          */
	ET1   = 0;                    /* Disable Timer1 Interrupt Enable bit.       */
	RXD   = 1;
	TXD   = 1;

	// Initialize Timer 2 to be the sampling rate
	T2CON = 0x00;
	RCAP2H = TIMER2_RELOAD_TOP;   /* Set low byte reload value  */
	RCAP2L = TIMER2_RELOAD_BOT;   /* Set High Byte reload value */
	TR2 = 1;                      /* start Timer 2              */

	// Setup the interrupts
	IE   |= 0xB0;	              /*
                                   * Enable global, serial, and timer 2 interrupts
	                               * or it so we don't mess with any other timer interrupts.
								   * that may have already been set.
								   */

	// Initialize the queue variables.
#if MAX_CMDS > 1
	cmd_head = cmd_tail = 0;
#else
	cmd_mutex = 0;
#endif
	
	//initialize channels
	voices[VOICE_0].wave_id = 1;
	voices[VOICE_1].wave_id = 2;
	voices[VOICE_2].wave_id = 3;
	voices[VOICE_3].wave_id = 4;

	//initialize voices
	channels[CHANNEL_0].position = 0;
	channels[CHANNEL_0].voice_id = VOICE_UNDEF;
	channels[CHANNEL_1].position = 0;
	channels[CHANNEL_1].voice_id = VOICE_UNDEF;
	channels[CHANNEL_2].position = 0;
	channels[CHANNEL_2].voice_id = VOICE_UNDEF;
	channels[CHANNEL_3].position = 0;
	channels[CHANNEL_3].voice_id = VOICE_UNDEF;


	// Initialize the Serial states and generate the music
	// bottom task.
	serial_state = SERIAL_STATE_START;
	new_serial_val = 0;
	os_create_task(MUSIC_BOT_TASK);

	// Serial Bottom loop.
	while (1) {
		INT1 = 0;
		T0 = 0;
		T1 = 0;
		while (!new_serial_val)
			os_wait(K_SIG, 0, 0);      // wait for a signal

		new_serial_val = 0;
		INT1 = 1;

		tmp = SBUF;

		switch (serial_state) {
		case SERIAL_STATE_START:
			temp_cmd.cmd_type = CMD_TYPE(tmp);
			temp_cmd.val = VAL(tmp);

			// Reset the temp_cmd struct fields.
			temp_cmd.id1 = 0;
			temp_cmd.note1 = 0;
			temp_cmd.id2 = 0;
			temp_cmd.note2 = 0;
			temp_cmd.id3 = 0;
			temp_cmd.note3 = 0;
			temp_cmd.id4 = 0;
			temp_cmd.note4 = 0;
			

			if (CMD_TYPE(tmp)) {
				serial_state = SERIAL_STATE_VOICE;
				num_read = 0;
			}
			else {
				T0 = 1;
				while (!enq_cmd()) {
					os_wait(K_SIG, 0, 0);
				}
				T0 = 0;
			}

			break;

		case SERIAL_STATE_VOICE:
			num_read++;
			
			if (num_read == 1) {
				temp_cmd.id1 = ID1_VAL(tmp);
				temp_cmd.note1 = NOTE1_VAL_UP(tmp);
			}
			else if (num_read == 2) {

				temp_cmd.id2 = ID2_VAL(tmp);
				temp_cmd.note1 += NOTE1_VAL_DWN(tmp);
				temp_cmd.note2 = NOTE2_VAL_UP(tmp);
			}
			else if (num_read == 3) {
				temp_cmd.id3 = ID3_VAL(tmp);
				temp_cmd.note2 += NOTE2_VAL_DWN(tmp);
				temp_cmd.note3 = NOTE3_VAL_UP(tmp);
			}
			else if (num_read == 4) {
				temp_cmd.id4 = ID4_VAL(tmp);
				temp_cmd.note3 += NOTE3_VAL_DWN(tmp);
			}
			else {
				temp_cmd.note4 = NOTE4_VAL(tmp);
				
				T1 = 1;
				while (!enq_cmd()) {
					os_wait(K_SIG, 0, 0);
				}
				T1 = 0;

				num_read = 0;

				serial_state = SERIAL_STATE_START;
			}

			break;

		default:
			serial_state = SERIAL_STATE_START;
			break;
		}

		// we finished processing the bytes, so send the acknowledgement.
		//SBUF = RETURN_CHAR;
		SBUF = tmp;
		P2 = tmp;
	}
}

//set the step based on a particular note that was passed in
//this also resets the index counters
void setStep(unsigned char idx, unsigned char index){
	channels[idx].stp.whole = steps_whole[index];
	channels[idx].stp.dec1 = steps_dec1[index];
	channels[idx].stp.dec1 = steps_dec2[index];
	
	channels[idx].stp.dec1_index = 0;
	channels[idx].stp.dec2_index = 0;
	channels[idx].stp.dec21_index = 0;
}


unsigned char getStepSize(unsigned char idx){
	//get the whole part of the step
	unsigned char temp1;
	unsigned char dec_place;
	temp1 = channels[idx].stp.whole;

	if(++channels[idx].stp.dec1_index > 9)
		channels[idx].stp.dec1_index = 0;

	dec_place = channels[idx].stp.dec1_index;

	switch(channels[idx].stp.dec1){
		case 0:
			temp1 += zero_tenth[dec_place];
			break;
		case 1:
			temp1 += one_tenth[dec_place];
			break;
		case 2:
			temp1 += two_tenth[dec_place];
			break;

		case 3:
			temp1 += three_tenth[dec_place];
			break;
		case 4:
			temp1 += four_tenth[dec_place];
			break;
		case 5:
			temp1 += five_tenth[dec_place];
			break;
		case 6:
			temp1 += six_tenth[dec_place];
			break;
		case 7:
			temp1 += seven_tenth[dec_place];
			break;
		case 8:
			temp1 += eight_tenth[dec_place];
			break;
		case 9:
			temp1 += nine_tenth[dec_place];
			break;
		 default:
		 	temp1 += zero_tenth[dec_place];
	}
	
	
	if(++channels[idx].stp.dec21_index > 9)
		channels[idx].stp.dec21_index = 0;

	if(one_tenth[channels[idx].stp.dec21_index]){


		if(++channels[idx].stp.dec2_index > 9)
			channels[idx].stp.dec2_index = 0;

		dec_place = channels[idx].stp.dec2_index;

		switch(channels[idx].stp.dec2){
		case 0:
			temp1 += zero_tenth[dec_place];
			break;
		case 1:
			temp1 += one_tenth[dec_place];
			break;
		case 2:
			temp1 += two_tenth[dec_place];
			break;

		case 3:
			temp1 += three_tenth[dec_place];
			break;
		case 4:
			temp1 += four_tenth[dec_place];
			break;
		case 5:
			temp1 += five_tenth[dec_place];
			break;
		case 6:
			temp1 += six_tenth[dec_place];
			break;
		case 7:
			temp1 += seven_tenth[dec_place];
			break;
		case 8:
			temp1 += eight_tenth[dec_place];
			break;
		case 9:
			temp1 += nine_tenth[dec_place];
			break;
		 default:
		 	temp1 += zero_tenth[dec_place];
		}

	}
	return temp1;
}

